/***************************************************************************
 *                          Zanoza Modeler v3.x                            *
 *      This is unpublished propietary source code of Zanoza Software.     *
 *      The copyright notice does not evidence any actual or intended      *
 *                    publication of such source code.                     *
 *                                                                         *
 *                Copyright (c) 2002-2012 Zanoza Software.                 *
 *                                                                         *
 *                        All rights reserved.                             *
 ***************************************************************************
 *--------------------------------------------------------------------------
 * @author:   Oleg M.
 *--------------------------------------------------------------------------
 * @purpose:  Mafia II vertices and indices data pool definition
 *--------------------------------------------------------------------------
 * @histroy: 08.09.2010. Created
 * @histroy: 07.12.2012. Remastering for ZModeler3
 */
#ifndef mafiadatapool_h_included
#define mafiadatapool_h_included
#include <zmodeler.h>
#include "FNV.h"
#include "float16.h"

namespace core
{
  namespace io
  {
#define MAFIAII_INDEX_POOL_ID           0x03
#define MAFIAII_VERTEX_POOL_ID          0x02

#pragma pack (push, 2)
    //
    // inidices pool starts with [BYTE] ID
    //
    struct  tIndicesBuffer
    {
      DWORD64     m_dwHash;     //hash on name of resource;
      long        m_nType;      //always 0x1 ?
      long        m_nSize;      //length of data;
      tUShortSeq  m_indices;
    };
    typedef ZSequence<tIndicesBuffer> tIndicesBufferSeq;

    struct tIndicesPool
    {
      long                m_nStreams;
      long                m_nVRAM;      //0x08-aligned data size includes all buffers{HASH, TYPE, LENGTH and DATA}
      tIndicesBufferSeq   m_Buffers;
    };


    //
    // vertices pool starts with [BYTE] ID
    //
    struct  tVerticesBuffer
    {
      DWORD64     m_dwHash;     //hash on name of resource;
      long        m_nSize;      //length of data;
      tBYTESeq    m_vertices;   //vertex structure defined outside and could include:
      // A float16 x,y,z        //word 0..0xffff decodes to float -1..+1
      // B byte[6] normals?     //0=?,1=?, x, -z, y, 5=?
      // C DWORD ? bone weights
      // D DWORD ? bone indices
      // E DWORD ? color mask. 0xFFFF0000 will paint to veh color; 0xFFFFFFFF will stay as is.
      // F DWORD ? properties? (glasses does not disappear when smashed if props set wrong).
      // G float16[2] uv0       //float 16;
      // H float16[2] uv1       //float 16;
      // I float16[2] uv2       //float 16;
      //
      // possible formats: 
      // 0x28 (40 bytes) ABCDEFGHI
      // 0x22 (36 bytes) ABCDEFGH
    };
    typedef ZSequence<tVerticesBuffer> tVerticesBufferSeq;

    struct tVerticesPool
    {
      long                m_nStreams;
      long                m_nVRAM;      //0x08-aligned data size includes all buffers{HASH, LENGTH and DATA}
      tVerticesBufferSeq  m_Buffers;
    };


#pragma pack (pop)

  };//io
};//core


#endif //#mafiadatapool_h_included

