/***************************************************************************
 *                          Zanoza Modeler v3.x                            *
 *      This is unpublished propietary source code of Zanoza Software.     *
 *      The copyright notice does not evidence any actual or intended      *
 *                    publication of such source code.                     *
 *                                                                         *
 *                Copyright (c) 2002-2012 Zanoza Software.                 *
 *                                                                         *
 *                        All rights reserved.                             *
 ***************************************************************************
 *--------------------------------------------------------------------------
 * @author:   Oleg M.
 *--------------------------------------------------------------------------
 * @purpose:  Mafia II filter header declarations
 *--------------------------------------------------------------------------
 * @histroy: 07.09.2010. Created
 * @histroy: 07.12.2012. Remastering for ZModeler3
 */
#ifndef mafiaii_h_included
#define mafiaii_h_included
#include <zmodeler.h>
#include <services/IImportExportService.h>
#include <helpers/services.h>
#include <helpers/layout.h>
#include <helpers/meshTools.h>
#include <helpers/nodes.h>
#include <helpers/undo.h>
#include <helpers/impexp.h>
#include <scene/IDummyNode.h>
#include <scene/IMeshNode.h>
//#include <scene/ISkeletonNode.h>
//#include <scene/IBoneNode.h>
#include <scene/IPolyMesh.h>
#include <scene/ITransformable.h>
#include <core/INamed.h>
#include "resource.h"
//
// mafia II data types
#include "MafiaDataPool.h"
#include "FrameResource.h"
#include "FrameNameTable.h"

namespace core
{
  namespace io
  {
    inline static float word2float(const WORD w, bool bCorr = false)
    {
      if (bCorr)
        return (float)(w >= 0x8000 ? w-32768.0f : w)/32768.0f;
      else
        return ((float)w - 32768.0f)/32768.0f;
    }

    inline static float byte2float(const BYTE b, bool bCorr = false)
    {
      return ((float)b - 127.0f)/128.0f;
    }


    //---------------------------------------------------------
    // @Name        : class CMafiaIIDialog
    // @Purpose     : Import/Export dialog for CMafiaII
    //---------------------------------------------------------
    class CMafiaII;
    class CMafiaIIDialog : public core::tools::CChildDialog
    {
    public:
      CMafiaIIDialog() : core::tools::CChildDialog() 
      {
        m_bExport = false;
        m_strDialogTitle  = _T("Mafia II Options");
      }
      DECLARE_MESSAGE_MAP()
    public:
      CHILDDIALOG_IMPLEMENT(CMafiaIIDialog, IDD_MAFIA)
      virtual ~CMafiaIIDialog();

      void          setDialogMode(bool b);
      bool          getDialogMode() {return m_bExport;}
      afx_msg void  OnDestroy();
      BOOL          OnInitDialog();
      void          setFileInfo(bool bLock, LPCTSTR pszComment);

    public:
      ZStdPtr<CMafiaII>         m_pOwner;
      bool                      m_bExport;
    };


    //--------------------------------------------------------
    //  Mafia II import/export filter
    //--------------------------------------------------------
    class CMafiaII :  public core::INamed,
                      public IImportExportFilter, 
                      public core::CErrorInfo
    {
      public:
        INTERFACE_DECLAREMAP(CMafiaII)

      public:
        CMafiaII();
        virtual ~CMafiaII();

      public:
      //-----------------------
      // core::INamed
      //-----------------------
        virtual ZRESULT getName(ZString& str);
        virtual ZRESULT setName(ZString str);
        virtual ZRESULT getTitle(ZString& str);
        virtual ZRESULT setTitle(ZString str);
        virtual ZRESULT getDescription(ZStringSeq& seqDesc);
        virtual ZRESULT setDescription(ZStringSeq& seqDesc);

      public:
      //-----------------------
      // IImportExportFilter
      //-----------------------
        virtual ZRESULT getExtensions(ZStringSeq& seqExt);
        virtual ZRESULT getExtensionSupport(LPCTSTR pszExt, DWORD& dwSupport);
        virtual ZRESULT setCurrentStream(core::io::IStream* pStream, LPCTSTR pszFileName=NULL, LPCTSTR pszPath=NULL);
        virtual ZRESULT readFileInfo(bool& bCanImport, tBYTESeq& seqThumbnailData);
        virtual ZRESULT getExportDialog(core::layout::IChildDialog** ppDlg, DWORD nMode=0xE);
        virtual ZRESULT getImportDialog(core::layout::IChildDialog** ppDlg, DWORD nMode=0xE);
        virtual ZRESULT enumImportableSet(long nWhich, ZStringSeq& seqSet) { return ZRESULT_FALSE; }
        virtual ZRESULT exportCurrentStream(core::IProcParams* pParams, DWORD nMode=0xE, ZBlackBox* pBox=NULL);
        virtual ZRESULT importCurrentStream(core::IProcParams* pParams, DWORD nMode=0xE, ZBlackBox* pBox=NULL);

      private:
      //-----------------------
      // implementation:
      //-----------------------
        ZRESULT attachSceneObject(scene::INode* pNode, 
                                  scene::INode* pParent, //NULL to attach to pFileRoot
                                  LPCTSTR pszName,
                                  ZMatrix& mat);
        ZRESULT createMeshNode( scene::IScene* pScene, 
                                scene::INode** ppNode, 
                                scene::IMeshNode** ppMNode, 
                                scene::IPolyMesh** ppMesh);
        ZRESULT createDummyNode(scene::INode*  pRoot, 
                                LPCTSTR pszName,
                                bool bSolid,
                                long nShape,
                                DWORD dwColor,
                                ZVector  vScale,
                                ZMatrix& matNode,
                                scene::IDummyNode** ppRes = NULL);
      private:
        // import-specific routines:
        ZRESULT importIndexBufferPool(tIndicesPool& pool, core::io::IStream* pStream, long nID = 0);
        ZRESULT importVertexBufferPool(tVerticesPool& pool, core::io::IStream* pStream, long nID = 0);
        ZRESULT importFrameResource(tFrameResource& frame, core::io::IStream* pStream, long nID = 0);
        ZRESULT importFrameNamesTable(tFrameNameTable& table, core::io::IStream* pStream, long nID = 0);
        ZRESULT readGeomDefinition(tGeometryModelDefinition& model, core::io::IStream* pStream);
        ZRESULT readSpacePartition(tSpacePartition& partition, core::io::IStream* pStream);
        ZRESULT readMaterialSplit(tMaterialSpltInfo& split, core::io::IStream* pStream, long nType);
        ZRESULT readModelsMaterialAssignment(tLODDescriptor& desc, core::io::IStream* pStream);
        ZRESULT readSceneCompositionSet(tSceneCompositionSet& sceneSet, core::io::IStream* pStream);
        ZRESULT readHierarchyAssembly(tHierarchyAssembly& hie, core::io::IStream* pStream);
        ZRESULT readParentingInfo(tHierarchyParenting& par, core::io::IStream* pStream);
        ZRESULT readCollisionBlocks(tFrameResource& frame, core::io::IStream* pStream);

        ZRESULT createScene(scene::IScene* pScene, tFrameNameTable& frameNameTable, 
                            tFrameResource& frame, tIndicesPool& ip, tVerticesPool& vp);

        // export-specific routines:

        //
        // helpers:
        scene::IDummyNode* locateSceneNode(scene::INode* pNode, LPCTSTR pszName);

      private:
      //-----------------------
      // data members:
      //-----------------------
        ZString                             m_strTitle;
        ZPtr<core::io::IStream>             m_pStream;
        ZPtr<services::ITexturesService>    m_pTexSrv;
        ZPtr<services::IMaterialsService>   m_pMatSrv;
        ZPtr<services::INodeService>        m_pNodeSrv;
        ZPtr<core::IProcParams>             m_pParams;
        ZPtr<scene::IScene>                 m_pScene;

        //import/export data members:

      private:
        ZPtr<scene::IDummyNode>             m_pRoot;

      public:
        //dialog
        ZPtr<CMafiaIIDialog>                m_pDlg;
        ZString                             m_strComment;
        ZString                             m_strFileName;//title only
        ZString                             m_strFilePath;//relative to specified file; includes trailing '\\'
        bool                                m_bLock;
        bool                                m_bThumb;
    };//CMafiaII

  };//io
};//core


#endif //mafiaii_h_included

